library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RAM_32x40_tb is
    --  Port ( );
end RAM_32x40_tb;

architecture Behavioral of RAM_32x40_tb is
    component RAM_32x40 is
        Port (
            clk     : in  std_logic;
            we      : in  std_logic;
            addrIn  : in  std_logic_vector (4 downto 0);
            addrOut : in  std_logic_vector (4 downto 0);
            dataIn  : in  std_logic_vector (0 to 39);
            dataOut : out std_logic_vector (0 to 39)
        );
    end component;
    
    signal clk: std_logic := '0';
    signal we : std_logic := '0';
    signal addrIn  : std_logic_vector (4 downto 0);
    signal addrOut : std_logic_vector (4 downto 0);
    signal dataIn  : std_logic_vector (0 to 39);
    signal dataOut : std_logic_vector (0 to 39);
    
    constant CLK_PERIOD : time := 10 ns;
    
begin
    UUT: RAM_32x40
        port map (
            clk                 => clk,
            we                  => we,
            addrIn(4 downto 0)  => addrIn(4 downto 0),
            addrOut(4 downto 0) => addrOut(4 downto 0),
            dataIn(0 to 39)     => dataIn(0 to 39),
            dataOut(0 to 39)    => dataOut(0 to 39)
        );
    
    STIM_CLK: process
    begin 
        wait for CLK_PERIOD/2;
        clk <= not clk;        
    end process;

    STIM: process 
    begin
        
        wait for CLK_PERIOD*2;
        
        -- initialize/write contents
        we <= '0';
        addrIn <= (others => '0');
        dataIn <= X"F0F0F0F0F0";
        for ii in 0 to 31 loop
            we <= '1';
            addrIn <= std_logic_vector(to_unsigned(ii,addrIn'length));
            wait for CLK_PERIOD;
            we <= '0';
            wait for CLK_PERIOD;
        end loop;
        
        
        -- read contents
        we <= '0';
        addrIn <= (others => '0');
        for ii in 0 to 31 loop
            addrOut <= std_logic_vector(to_unsigned(ii,addrOut'length));
            wait for CLK_PERIOD/4;            
        end loop;
        
        
        -- write and read
        we <= '0';
        addrIn <= (others => '0');
        dataIn <= (others => '0');
        wait for CLK_PERIOD;
        
        for ii in 0 to 31 loop            
            we <= '1';
            addrIn <= std_logic_vector(to_unsigned(ii,addrIn'length));
            dataIn <= std_logic_vector(to_unsigned(ii,dataIn'length));
            wait for CLK_PERIOD;
            we <= '0';
            addrOut <= std_logic_vector(to_unsigned(ii,addrOut'length));
            wait for CLK_PERIOD;
        end loop;
        
        wait;
    end process;

end Behavioral;
