----------------------------------------------------------------------------------
-- RAM: 32 vrstic po 40 bitov
-- Dvokanalni: 
--     - branje in pisanje se izvajata istoasno
--     - asinhron bralni dostop: ko damo naslov na addrOut, dobimo podatke na dataOut
--     - sinhron pisalni dostop: podatki na dataIn se zapiejo na naslov addrIn 
--                               ob urini fronti in visokem write-enable (we) 
-- Aplikacija: slikovni medpomnilnik za krmilnik VGA (frame buffer)
--     - za nas so zanimivi naslovi od 0 do 29, torej 30 vrstic po 40 bitov, saj
--       gre za poenostavitev in je pomnilnik velik 1/16 osnovne resolucije VGA:
--       (30x40)*16 = 480x640
--     - pozor: biti v eni vrstici so orientirani od LSB -> MSB, 
--              da se bolj ujema z organizacijo zaslona
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RAM_32x40 is
    Port (
        clk     : in  std_logic;
        we      : in  std_logic;
        addrIn  : in  std_logic_vector (4 downto 0);
        addrOut : in  std_logic_vector (4 downto 0);
        dataIn  : in  std_logic_vector (0 to 39);
        dataOut : out std_logic_vector (0 to 39)
    );
end entity;

architecture Behavioral of RAM_32x40 is
    -- Definiramo polje pomnilnikih besed (polje vrstic)
    -- Vrstica je zapisana v smeri od najmanj do najbolj pomembnega bita (LSB bit je skrajno levi)
    type RAM_type is array (0 to 31) of std_logic_vector(0 to 39);
    signal RAM : RAM_type; --:= (others => (others => '0'));

begin
    -- asinhrono branje; naslov, ki je tipa std_logic_vector moramo pretvoriti v integer
    dataOut <= RAM(to_integer(unsigned(addrOut)));

    -- sinhrono pisanje
    SYNC_PROC: process (clk)
    begin
        if rising_edge(clk) then
            if we='1' then
                RAM(to_integer(unsigned(addrIn))) <= dataIn;
            end if;
        end if;
    end process;
end Behavioral;
